<?php

namespace App\Http\Controllers;

use App\Models\User;
use Illuminate\Support\Facades\Validator;
use Omnipay\Omnipay;
use App\Models\Payment;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use phpDocumentor\Reflection\PseudoTypes\True_;

class PaymentController extends Controller
{

    public $gateway;

    public function __construct()
    {
        $this->gateway = Omnipay::create('PayPal_Rest');
        $this->gateway->setClientId(env('PAYPAL_CLIENT_ID'));
        $this->gateway->setSecret(env('PAYPAL_CLIENT_SECRET'));
        $this->gateway->setTestMode(true); //set it to 'false' when go live
    }

    public function index()
    {
        return view('payment');
    }

    public function charge(Request $request)
    {
        if($request->payment_type=='paypal')
        {
            try {
                $response = $this->gateway->purchase(array(
                    'amount' => session('paypal'),
                    'currency' => env('PAYPAL_CURRENCY'),
                    'returnUrl' => url('paymentsuccess'),
                    'cancelUrl' => url('paymenterror'),
                ))->send();

                if ($response->isRedirect()) {
                    $response->redirect(); // this will automatically forward the customer
                } else {
                    // not successful
                    return $response->getMessage();
                }
            } catch(Exception $e) {
                return $e->getMessage();
            }
        }
        else{
            if (session('orderLater')[1]['date']==NULL && session('orderLater')[1]['time']==NULL){
                $delivery = 1;
                $del_date = Carbon::now();
                $del_time = Carbon::now();
            }
            else{
                $delivery = 0;
                $date=date_create(session('orderLater')[1]['date']);
                $del_date = date_format($date,'Y-m-d');
                $del_time = date("H:i:s", strtotime( session('orderLater')[1]['time'] ));
            }
            $validator = Validator::make($request->all(), [
                'client_address' => 'required',
                'payment_type' => 'required',
            ]);

            if ($validator->fails()) {
                return redirect()->back()
                    ->withErrors($validator)
                    ->withInput();
            }
            $orderID =  DB::table('orders')->insertGetId([
                'order_unique_id'=>uniqid(),
                'total_amount'=>session('cod'),
                'delivery_charge'=>0,
                'payment_type'=>'cod',
                'sub_total'=>session('cod'),
                'discount_in_percent'=>0,
                'promo_code'=>NULL,
                'payment_status'=>'new',
                'notes'=>NULL,
                'client_id'=>Auth::user()->id,
                'is_delivery_now'=>$delivery,
                'address1'=>$request->client_address,
                'address2'=>NULL,
                'building_and_unit_number'=>NULL,
                'delivery_address'=>session()->get('isLocationSelected'),
                'date_deliver'=>$del_date,
                'time_deliver'=>$del_time,
                'date_receive'=>Carbon::now(),
                'is_paid'=> 0,
            ]);
            foreach(session('cart') as $key=>$cart){
            DB::table('order_items')->insert([
                'product_item_id'=>$key,
                'product_item_count'=>$cart['quantity'],
                'order_id'=>$orderID,
            ]);
            if(!session('product'.$key)==NULL){
                foreach(session('product'.$key) as $ingredients){
                    DB::table('order_item_ingredients')->insert([
                        'product_item_id'=>$key,
                        'product_ingredient_id'=>$ingredients['ingredient_id'],
                        'order_id'=>$orderID
                    ]);
                }
            }
            }
            $user = User::find(Auth::user()->id);
            $user->no_of_orders = (@$user->no_of_orders) ? @$user->no_of_orders + 1 : 1;
            $user->total_spent = $user->total_spent + session('cod');
            $user->save();

            session()->forget('orderLater');
            session()->forget('isLocationSelected');
            session()->forget('cart');
            session()->forget('promo_discount');
            return redirect()->route('cod');
        }
    }

    public function payment_success(Request $request)
    {
        // Once the transaction has been approved, we need to complete it.
        if ($request->input('paymentId') && $request->input('PayerID'))
        {
            $transaction = $this->gateway->completePurchase(array(
                'payer_id'             => $request->input('PayerID'),
                'transactionReference' => $request->input('paymentId'),
            ));
            $response = $transaction->send();

            if ($response->isSuccessful())
            {
                // The customer has successfully paid.
                $arr_body = $response->getData();

                // Insert transaction data into the database
                $isPaymentExist = Payment::where('payment_id', $arr_body['id'])->first();

                if(!$isPaymentExist)
                {
                    $payment = new Payment;
                    $payment->payment_id = $arr_body['id'];
                    $payment->payer_id = $arr_body['payer']['payer_info']['payer_id'];
                    $payment->payer_email = $arr_body['payer']['payer_info']['email'];
                    $payment->amount = $arr_body['transactions'][0]['amount']['total'];
                    $payment->currency = env('PAYPAL_CURRENCY');
                    $payment->payment_status = $arr_body['state'];
                    $payment->save();
                }
//                foreach (session('cart') as $cart){
//                    echo $cart['price'];
//                }
                if (session('orderLater')[1]['date']==NULL && session('orderLater')[1]['time']==NULL){
                    $delivery = 1;
                    $del_date = Carbon::now();
                    $del_time = Carbon::now();
                }
                else{
                    $delivery = 0;
                    $date=date_create(session('orderLater')[1]['date']);
                    $del_date = date_format($date,'Y-m-d');
                    $del_time = date("H:i:s", strtotime( session('orderLater')[1]['time'] ));
                }
               $orderID =  DB::table('orders')->insertGetId([
                    'order_unique_id'=>uniqid(),
                    'total_amount'=>$arr_body['transactions'][0]['amount']['total'],
                    'delivery_charge'=>0,
                    'payment_type'=>'paypal',
                    'sub_total'=>session('cod'),
                    'discount_in_percent'=>0,
                    'promo_code'=>NULL,
                    'payment_status'=>'new',
                    'notes'=>NULL,
                    'client_id'=>Auth::user()->id,
                    'is_delivery_now'=>$delivery,
                    'address1'=>$request->client_address,
                    'address2'=>NULL,
                    'building_and_unit_number'=>NULL,
                    'delivery_address'=>session()->get('isLocationSelected'),
                    'date_deliver'=>$del_date,
                    'time_deliver'=>$del_time,
                    'date_receive'=>Carbon::now(),
                    'is_paid'=> 1,
                ]);
                foreach(session('cart') as $key=>$cart){
                DB::table('order_items')->insert([
                    'product_item_id'=>$key,
                    'product_item_count'=>$cart['quantity'],
                    'order_id'=>$orderID,
                ]);
                if(!session('product'.$key)==NULL){
                    foreach(session('product'.$key) as $ingredients){
                        DB::table('order_item_ingredients')->insert([
                            'product_item_id'=>$key,
                            'product_ingredient_id'=>$ingredients['ingredient_id'],
                            'order_id'=>$orderID
                        ]);
                    }
                }
                }
                $user = User::find(Auth::user()->id);
                $user->no_of_orders = (@$user->no_of_orders) ? @$user->no_of_orders + 1 : 1;
                $user->total_spent = $user->total_spent + session('cod');
                $user->save();

                session()->forget('isLocationSelected');
                session()->forget('orderLater');
                session()->forget('cart');
                session()->forget('promo_discount');
                return view('payment_successfully')->with('tran_id',$arr_body['id']);
            } else {
                return $response->getMessage();
            }
        } else {
            return view('payment_cancel');
        }
    }

    public function payment_error()
    {
        return view('payment_cancel');
    }

}
