<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\ClientResource;
use Illuminate\Http\Request;
use Carbon\Carbon;
use App\User;
use App\failed_payment_error_log;
use Illuminate\Support\Facades\DB;
use BrowserDetect;
class ClientsController extends Controller
{
    /**
     * Get client records
     *
     * @return  ClientResource  client resource data with pagination
     */
    public function index()
    {
        if (!empty($_GET['city'])) {
            if ($_GET['city'] == 'asc' || $_GET['city'] == 'desc') {
                $clients = User::where('is_admin', '0')->orderBy('city', $_GET['city'])->paginate(10);

                return ClientResource::collection($clients);
            }
        }

        if (!empty($_GEt['order'])) {
            if ($_GEt['order'] == 'asc' || $_GEt['order'] == 'desc') {
                $clients = User::where('is_admin', '0')->orderBy('no_of_order', $_GET['order'])->paginate(10);

                return ClientResource::collection($clients);
            }
        }

        $clients = User::where('is_admin', '0')->paginate(10);

        return ClientResource::collection($clients);
    }

    /**
     * Get client data by ID
     *
     * @param   int  $id  client ID
     *
     * @return  JSON      client data or message
     */
    public function show($id)
    {
        try {
            $client = User::with(['orders' => function ($q) use ($id) {
                $q->with(['orderItems' => function ($q1) {
                    $q1->with(['product'])->get();
                }])->get();
            }])->where('is_admin', '0')->findOrFail($id);

            return response()->json($client, 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Client data not found.'
            ], 404);
        }
    }

    /**
     * Get total clients, current and previous months clients count
     *
     * @return  JSON      client count data
     */
    public function clientsStats()
    {
        $currStartMonth = Carbon::today()->startOfMonth();
        $currEndMonth = Carbon::today()->endOfMonth();
        $prevStartMonth = Carbon::now()->subMonth()->startOfMonth();
        $prevEndMonth = Carbon::now()->subMonth()->endOfMonth();

        $clientsCount = User::client()->count();
        $clientsCurrMonthCount = User::client()->whereBetween('created_at', [$currStartMonth, $currEndMonth])->count();
        $clientsPrevMonthCount = User::client()->whereBetween('created_at', [$prevStartMonth, $prevEndMonth])->count();

        $clientData = new \stdClass();
        $clientData->count = $clientsCount;
        $clientData->curr_month_count = $clientsCurrMonthCount;
        $clientData->prev_month_count = $clientsPrevMonthCount;
        $clientData->curr_percentage = $clientsCurrMonthCount / $clientsCount * 100;
        $clientData->prev_percentage = $clientsPrevMonthCount / $clientsCount * 100;

        return response()->json($clientData, 200);
    }
    public function updateClientAccount(Request $request)
    {

        if (@$request->profile_pic != "") {
            $image      = $request->file('profile_pic');
            $fileName   = time() . '.' . $image->getClientOriginalExtension();

            $img = \Image::make($image->getRealPath());
            $img->resize(120, 120, function ($constraint) {
                $constraint->aspectRatio();
            });
            $img->stream();
        }
        $user  = new  User;
        $user::where('id', $request->id)
            ->update([
                'first_name' => $request->first_name,
                'last_name' => $request->last_name,
                'phone' => $request->phone,
                'email' => $request->email,
                'profile_pic' => (@$request->profile_pic != "") ? 'images/profile_picture/user_id_' . $request->id . '/' . $fileName : $request->old_profile_pic,
                'address' => $request->address,
                'city' => $request->city,
                'country' => $request->country,
            ]);

        if (@$request->profile_pic != "") {
            \Storage::disk('uploads')->put('profile_picture/user_id_' . $request->id . '/' . $fileName, $img, 'public');
        }

        $user  = new  User;
        $user = $user::where('id', $request->id)->first();
        $tokenResult = $user->createToken('MySushi');
        $token = $tokenResult->token;

        if ($request->remember_me) {
            $token->expires_at = Carbon::now()->addWeeks(1);
        }
        $token->save();
        return response()->json([
            'access_token' => $tokenResult->accessToken,
            'token_type' => 'Bearer',
            'expires_at' => Carbon::parse(
                $tokenResult->token->expires_at
            )->toDateTimeString(),
            'message' => 'Authenticated.',
            'user' => $user,
            'code' => '200'
        ]);
    }
    public function saveAddress(Request $request)
    {
        //  $User =  New User;

        $user = User::where('id', $request->user_id)
            ->update([
                'name' => $request->name,
                'address1' => $request->address1,
                'address2' => $request->address2,
                'building_and_unit_number' => $request->building_and_unit_number,
                'phone' => $request->phone,
            ]);
        return $request->user_id;
    }

    public function PaypalPaymentLogs($response , $order_id , $payment_type , $log_status , $log_type){
        $error_log = new failed_payment_error_log();

        $platform_name = BrowserDetect::platformName();
        $browser_name =' | ' . BrowserDetect::browserName();
        $device_name = ' | ' . BrowserDetect::deviceFamily();
        $device_model =' | ' . BrowserDetect::deviceModel();

        $error_log->user_agent = $platform_name.$browser_name.$device_name.$device_model;
        $error_log->log_status = $log_status;
        $error_log->response = $response;
        $error_log->order_id = $order_id;
        $error_log->log_type = $log_type;
        if($payment_type == 1){
          $error_log->payment_type = 'paypal';
        }else{
          $error_log->payment_type = 'cod';
        }


        $error_log->save();

        return response()->json('success' , 200);
    }
}
