<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\IngredientsCategoryRequest;
use App\Http\Resources\IngredientCategoryResource;
use App\IngredientCategory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Intervention\Image\Facades\Image;
use File;
class IngredientCategoryController extends Controller
{
    public function index(){
        $ingredient = IngredientCategory::paginate(100);

        return IngredientCategoryResource::collection($ingredient);
    }
    public function show($id)
    {
        try {
            $ingredient = IngredientCategory::findOrFail($id);

            return response()->json($ingredient, 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Product item not found.'
            ], 404);
        }
    }
    public function status_change($id){
        $ingredients = IngredientCategory::findOrFail($id);

        if ($ingredients->status==0){
            try {
                DB::beginTransaction();
                $ingredients->status = 1;
                $ingredients->save();
            } catch (QueryException $qe) {
                DB::rollback();
                return $qe;
            }
        }
        else{
            try {
                DB::beginTransaction();
                $ingredients->status = 0;
                $ingredients->save();
            } catch (QueryException $qe) {
                DB::rollback();
                return $qe;
            }
        }
        return response()->json([
            'success' => true,
            'data1' => $ingredients,
        ], 200);

    }
    public function store(IngredientsCategoryRequest $request)
    {
        $ingredient = new IngredientCategory();

        if (!empty($request->ingredientsCategory)) {
            $request->ingredientsCategory = json_decode($request->ingredientsCategory);
        } else {
            return response()->json([
                'message' => 'Ingredient data empty.'
            ], 422);
        }

        try {
            DB::beginTransaction();
            $ingredient->name = $request->ingredientsCategory->ingredientsCategory_name;
            $ingredient->status = 1;
            $ingredient->save();

            if ($request->hasFile('ingredientsCategory_desktop_pic')) {
                $desktopPicFile = $request->file('ingredientsCategory_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt  =   pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredientsCategory/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredient->id . '.' . $desktopPicFileExt);
                $itemDesktopPic = $ingredient->id . '.' . $desktopPicFileExt;
                Image::make($request->file('ingredientsCategory_desktop_pic')->getRealPath())->save($path . $itemDesktopPic);
            }

            $updateProduct = IngredientCategory::findOrFail($ingredient->id);
            $updateProduct->desktop_pic = (!empty($itemDesktopPic)) ? $itemDesktopPic : null;
            $updateProduct->save();

            if ($updateProduct->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $ingredient,
                    'data2' => $updateProduct,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error saving product.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('ingredientsCategory_desktop_pic')) {
                $file = public_path() . '/img/ingredientsCategory/' . $ingredient->desktop_pic;
                File::delete($file);
            }


            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            if ($request->hasFile('ingredientsCategory_desktop_pic')) {
                $file = public_path() . '/img/ingredientsCategory/' . $ingredient->desktop_pic;
                File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }
    public function update(IngredientsCategoryRequest $request, $id)
    {
        $ingredients = IngredientCategory::findOrFail($id);

        if (!empty($request->ingredientsCategory)) {
            $request->ingredientsCategory = json_decode($request->ingredientsCategory);
        } else {
            return response()->json([
                'message' => 'Item data enpty.'
            ], 422);
        }

        try {
            DB::beginTransaction();
            $ingredients->name = $request->ingredientsCategory->ingredientsCategory_name;
            $ingredients->save();

            if ($request->hasFile('ingredientsCategory_desktop_pic') && !empty($request->file('ingredientsCategory_desktop_pic'))) {
                $desktopPicFile = $request->file('ingredientsCategory_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt  =   pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredientsCategory/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredients->id . '.' . $desktopPicFileExt);
                $itemDesktopPic = $ingredients->id . '.' . $desktopPicFileExt;
                Image::make($request->file('ingredientsCategory_desktop_pic')->getRealPath())->save($path . $itemDesktopPic);
            }


            $updateingredients = IngredientCategory::findOrFail($ingredients->id);
            $updateingredients->desktop_pic = (!empty($itemDesktopPic)) ? $itemDesktopPic : $ingredients->desktop_pic;
            $updateingredients->save();

            if ($updateingredients->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $ingredients,
                    'data2' => $updateingredients,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error updating product.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('ingredientsCategory_desktop_pic')) {
                $file = public_path() . '/img/ingredientsCategory/' . $ingredients->desktop_pic;
                \File::delete($file);
            }


            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            if ($request->hasFile('ingredientsCategory_desktop_pic')) {
                $file = public_path() . '/img/ingredientsCategory/' . $ingredients->desktop_pic;
                \File::delete($file);
            }


            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    public function destroy($id)
    {
        try {
            DB::beginTransaction();

            $productCategory = IngredientCategory::findOrFail($id);

            if ($productCategory->delete()) {
                DB::commit();

                $desktopImgFile = public_path() . '/img/ingredientsCategory/' . $productCategory->desktop_pic;
                \File::delete($desktopImgFile);

                $mobileImgFile = public_path() . '/img/ingredientsCategory/' . $productCategory->mobile_pic;
                \File::delete($mobileImgFile);

                return response()->json([
                    'success' => true,
                    'message' => 'Ingredients category deleted.'
                ], 202);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (\Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => 'Ingredients category not found.'
            ], 404);
        }
    }
}
