<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\IngredientsRequest;
use App\Http\Resources\IngredientControllerResource;
use App\Http\Resources\IngredientResource;
use App\Ingredient;
use App\IngredientCategory;
use App\IngredientProduct;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use File;
use Illuminate\Support\Facades\Schema;
use Intervention\Image\Facades\Image;

class IngredientsController extends Controller
{

    public function index()
    {
        $ingredient = Ingredient::with('products')->paginate(100);

        return IngredientResource::collection($ingredient);
    }

    public function IngredientsProducts(){
        $ingredient = IngredientProduct::with('product')->get();
        return IngredientControllerResource::collection($ingredient);
    }

    public function status_change($id){
        $ingredients = Ingredient::findOrFail($id);

        if ($ingredients->status==0){
            try {
                DB::beginTransaction();
                $ingredients->status = 1;
                $ingredients->save();
                DB::table('ingredient_products')->where('ingredient_id', $id)->update([
                    'status'=>1
                ]);
            } catch (QueryException $qe) {
                DB::rollback();
                return $qe;
            }
        }
        else{
            try {
                DB::beginTransaction();
                $ingredients->status = 0;
                $ingredients->save();
                DB::table('ingredient_products')->where('ingredient_id', $id)->update([
                    'status'=>0
                ]);
            } catch (QueryException $qe) {
                DB::rollback();
                return $qe;
            }
        }
        return response()->json([
            'success' => true,
            'data1' => $ingredients,
        ], 200);

    }

    public function store(IngredientsRequest $request)
    {
        $ingredient = new Ingredient();

        if (!empty($request->ingredients)) {
            $request->ingredients = json_decode($request->ingredients);
        } else {
            return response()->json([
                'message' => 'Ingredient data empty.'
            ], 422);
        }
        $elementCount  = count($request->ingredients->ingredients_product);

        try {
            DB::beginTransaction();
            $ingredient->name = $request->ingredients->ingredients_name;
            $ingredient->price = $request->ingredients->ingredients_price;
            $ingredient->stock = $request->ingredients->ingredients_stock;
            $ingredient->status = 1;
            $ingredient->ingredients_category = $request->ingredients->ingredient_category;
            $ingredient->save();

            for ($i = 0; $i < $elementCount; $i ++) {
                    DB::table('ingredient_products')->insert([
                        'product_id'=> $request->ingredients->ingredients_product[$i]->id,
                        'ingredient_id'=>$ingredient->id,
                        'status'=>1
                    ]);
            }
            if ($request->hasFile('ingredients_desktop_pic')) {
                $desktopPicFile = $request->file('ingredients_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt  =   pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredients/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredient->id . '.' . $desktopPicFileExt);
                $itemDesktopPic = $ingredient->id . '.' . $desktopPicFileExt;
                Image::make($request->file('ingredients_desktop_pic')->getRealPath())->save($path . $itemDesktopPic);
            }


            if ($request->hasFile('ingredients_mobile_pic')) {
                $mobilePicFile = $request->file('ingredients_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt  =   pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredients/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredient->id . '_mobile.' . $mobilePicFileExt);
                $itemMobilePic = $ingredient->id . '_mobile.' . $mobilePicFileExt;
                Image::make($request->file('ingredients_mobile_pic')->getRealPath())->save($path . $itemMobilePic);
            }


            if ($request->hasFile('ingredients_header_pic')) {
                $headerPicFile = $request->file('ingredients_header_pic');
                $headerPicFileName = $headerPicFile->getClientOriginalName();
                $headerPicFileExt  =   pathinfo($headerPicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredients/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredient->id . '_item_header.' . $headerPicFileExt);
                $itemHeaderPic = $ingredient->id . '_item_header.' . $headerPicFileExt;
                Image::make($request->file('ingredients_header_pic')->getRealPath())->save($path . $itemHeaderPic);
            }

            $updateProduct = Ingredient::findOrFail($ingredient->id);
            $updateProduct->desktop_pic = (!empty($itemDesktopPic)) ? $itemDesktopPic : null;
            $updateProduct->mobile_pic = (!empty($itemMobilePic)) ? $itemMobilePic : null;
            $updateProduct->header_pic = (!empty($itemHeaderPic)) ? $itemHeaderPic : null;
            $updateProduct->save();

            if ($updateProduct->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $ingredient,
                    'data2' => $updateProduct,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error saving product.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('item_desktop_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredient->desktop_pic;
                File::delete($file);
            }

            if ($request->hasFile('item_mobile_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredient->mobile_pic;
                File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            if ($request->hasFile('item_desktop_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredient->desktop_pic;
                File::delete($file);
            }

            if ($request->hasFile('item_mobile_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredient->mobile_pic;
                File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }
    public function show($id)
    {
        try {
            $ingredient = Ingredient::with(['products'])->findOrFail($id);

            return response()->json($ingredient, 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Product item not found.'
            ], 404);
        }
    }
    public function update(IngredientsRequest $request, $id)
    {
        $ingredients = Ingredient::findOrFail($id);

        if (!empty($request->ingredients)) {
            $request->ingredients = json_decode($request->ingredients);
        } else {
            return response()->json([
                'message' => 'Item data enpty.'
            ], 422);
        }

        try {
            DB::beginTransaction();
            $ingredients->name = $request->ingredients->ingredients_name;
            $ingredients->product_id = $request->ingredients->ingredients_product;
            $ingredients->price = $request->ingredients->ingredients_price;
            $ingredients->stock = $request->ingredients->ingredients_stock;
//            $ingredients->ingredients_category = $request->ingredients->ingredient_category;
            $ingredients->save();

            if ($request->hasFile('ingredients_desktop_pic') && !empty($request->file('ingredients_desktop_pic'))) {
                $desktopPicFile = $request->file('ingredients_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt  =   pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredients/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredients->id . '.' . $desktopPicFileExt);
                $itemDesktopPic = $ingredients->id . '.' . $desktopPicFileExt;
                Image::make($request->file('ingredients_desktop_pic')->getRealPath())->save($path . $itemDesktopPic);
            }

            if ($request->hasFile('ingredients_mobile_pic') && !empty($request->file('ingredients_mobile_pic'))) {
                $mobilePicFile = $request->file('ingredients_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt  =   pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredients/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredients->id . '_mobile.' . $mobilePicFileExt);
                $itemMobilePic = $ingredients->id . '_mobile.' . $mobilePicFileExt;
                Image::make($request->file('ingredients_mobile_pic')->getRealPath())->save($path . $itemMobilePic);
            }
            if ($request->hasFile('ingredients_header_pic') && !empty($request->file('ingredients_header_pic'))) {
                $headerPicFile = $request->file('ingredients_header_pic');
                $headerPicFileName = $headerPicFile->getClientOriginalName();
                $headerPicFileExt  =   pathinfo($headerPicFileName, PATHINFO_EXTENSION);

                $path = 'img/ingredients/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $ingredients->id . '_item_header.' . $headerPicFileExt);
                $itemHeaderPic = $ingredients->id . '_item_header.' . $headerPicFileExt;
                Image::make($request->file('ingredients_header_pic')->getRealPath())->save($path . $itemHeaderPic);
            }

            $updateingredients = Ingredient::findOrFail($ingredients->id);
            $updateingredients->desktop_pic = (!empty($itemDesktopPic)) ? $itemDesktopPic : $ingredients->desktop_pic;
            $updateingredients->mobile_pic = (!empty($itemMobilePic)) ? $itemMobilePic : $ingredients->mobile_pic;
            $updateingredients->header_pic = (!empty($itemHeaderPic)) ? $itemHeaderPic : $ingredients->header_pic;
            $updateingredients->save();

            if ($updateingredients->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $ingredients,
                    'data2' => $updateingredients,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error updating product.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('ingredients_desktop_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredients->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('ingredients_mobile_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredients->mobile_pic;
                \File::delete($file);
            }
            if ($request->hasFile('ingredients_header_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredients->header_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            if ($request->hasFile('ingredients_desktop_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredients->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('ingredients_mobile_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredients->mobile_pic;
                \File::delete($file);
            }
            if ($request->hasFile('ingredients_header_pic')) {
                $file = public_path() . '/img/ingredients/' . $ingredients->header_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }
    public function destroy($id)
    {
        try {
            DB::beginTransaction();

            $productCategory = Ingredient::findOrFail($id);

            if ($productCategory->delete()) {
                DB::commit();

                $desktopImgFile = public_path() . '/img/ingredients/' . $productCategory->desktop_pic;
                \File::delete($desktopImgFile);

                $mobileImgFile = public_path() . '/img/ingredients/' . $productCategory->mobile_pic;
                \File::delete($mobileImgFile);

                return response()->json([
                    'success' => true,
                    'message' => 'Ingredient deleted.'
                ], 202);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (\Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => 'Ingredient not found.'
            ], 404);
        }
    }

}
