<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\LoyaltyProgram;
use App\Models\LoyaltyOffer;
use App\Models\ProductCategory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\File;

class LoyaltyController extends Controller
{
    public function store_program(Request $request){
      $data = json_decode($request->program);
      $program = new LoyaltyProgram();

      $program->name = $data->name;
      $program->min_points = $data->min_points;
      $program->max_points = $data->max_points;
      $program->is_featured = '0';
      $program->offers_selected = json_encode($data->offers_selected);

      $desktop_path ='';
      if ($request->hasFile('desktop_image') && !empty($request->file('desktop_image'))) {
        $homeimg = $request->file('desktop_image');
        $unique_id = substr(microtime(false), 2, 8);
        $desktop_path = 'loyalty/'.$unique_id.'.'.$homeimg->getClientOriginalExtension();
        Storage::disk('frontend')->put($desktop_path,  File::get($homeimg));
      }
      $mobile_path ='';
      if ($request->hasFile('mobile_image') && !empty($request->file('mobile_image'))) {
        $mobileImg = $request->file('mobile_image');
        $unique_id = substr(microtime(false), 2, 8);
        $mobile_path = 'loyalty/'.$unique_id.'.'.$mobileImg->getClientOriginalExtension();
        Storage::disk('frontend')->put($mobile_path,  File::get($mobileImg));
      }

      $desktop_path = 'img/'.$desktop_path;
      $mobile_path = 'img/'.$mobile_path;

      $program->mobile_image = $mobile_path;
      $program->desktop_image = $desktop_path;

      if($program->save()){
        return response()->json(['status' => 200]);
      }
      return response()->json(['status' => 500]);
    }

    public function update_program(Request $request){
      $data = json_decode($request->program);
      $program = LoyaltyProgram::find($data->id);

      if($program){
        $program->name = $data->name;
        $program->min_points = $data->min_points;
        $program->max_points = $data->max_points;
        $program->is_featured = '0';
        $program->offers_selected = json_encode($data->offers_selected);
      }

      $desktop_path ='';
      if ($request->hasFile('desktop_image') && !empty($request->file('desktop_image'))) {
        $homeimg = $request->file('desktop_image');
        $unique_id = substr(microtime(false), 2, 8);
        $desktop_path = 'loyalty/'.$unique_id.'.'.$homeimg->getClientOriginalExtension();
        Storage::disk('frontend')->put($desktop_path,  File::get($homeimg));
        $desktop_path = 'img/'.$desktop_path;
        $program->desktop_image = $desktop_path;
      }
      $mobile_path ='';
      if ($request->hasFile('mobile_image') && !empty($request->file('mobile_image'))) {
        $mobileImg = $request->file('mobile_image');
        $unique_id = substr(microtime(false), 2, 8);
        $mobile_path = 'loyalty/'.$unique_id.'.'.$mobileImg->getClientOriginalExtension();
        Storage::disk('frontend')->put($mobile_path,  File::get($mobileImg));
        $mobile_path = 'img/'.$mobile_path;
        $program->mobile_image = $mobile_path;
      }

      if($program->update()){
        return response()->json(['status' => 200]);
      }
      return response()->json(['status' => 500]);
    }

    public function get_programs($type){
      if($type == 'all'){
        $offers = LoyaltyProgram::all();
        return response()->json($offers, 200);
      }
    }

    public function get_single_program($id){
    $program = LoyaltyProgram::find($id);
    if($program->offers_selected != ''){
      $offers = LoyaltyOffer::whereIn('id',json_decode($program->offers_selected))->get();
      $program->offers_selected = $offers;
    }
    return response()->json($program, 200);
  }

    public function set_featured($id, $status){
      try {
        DB::beginTransaction();

        if ($status === null) {
          return response()->json([
              'success' => false,
              'message' => 'Promo ID active status not found.'
          ], 422);
        }

        $program = LoyaltyProgram::findOrFail($id);
        $program->is_featured = $status;

        if ($program->save()) {
          DB::commit();

          return response()->json([
              'success' => true,
              'data' => $program
          ], 200);
        } else {
          return response()->json([
              'success' => false,
              'message' => 'Error changing program featured status.'
          ], 404);
        }
      } catch(QueryException $qe) {
        DB::rollback();

        return response()->json([
            'success' => false,
            'message' => $qe->getMessage()
        ], 500);
      } catch(\Exception $e) {
        DB::rollback();

        return response()->json([
            'success' => false,
            'message' => 'Program not found.'
        ], 404);
      }
    }

    public function set_status($id, $status){
      try {
        DB::beginTransaction();

        if ($status === null) {
          return response()->json([
              'success' => false,
              'message' => 'Promo ID active status not found.'
          ], 422);
        }

        $program = LoyaltyProgram::findOrFail($id);
        $program->status = $status;

        if ($program->save()) {
          DB::commit();

          return response()->json([
              'success' => true,
              'data' => $program
          ], 200);
        } else {
          return response()->json([
              'success' => false,
              'message' => 'Error changing program status.'
          ], 404);
        }
      } catch(QueryException $qe) {
        DB::rollback();

        return response()->json([
            'success' => false,
            'message' => $qe->getMessage()
        ], 500);
      } catch(\Exception $e) {
        DB::rollback();

        return response()->json([
            'success' => false,
            'message' => 'Program not found.'
        ], 404);
      }
    }

    public function store_offer(Request $request){
      $data = json_decode($request->offer);
      $offer = new LoyaltyOffer();

      $offer->name = $data->name;
      $offer->type = $data->type;
      if($data->type == 'off'){
        $offer->off_amount = $data->off_amount;
        $offer->off_order = $data->off_order;
      }elseif ($data->type == 'birthday_gift' || $data->type == 'christmas_gift' || $data->type == 'anniversary_gift' ){
        $offer->categories_selected = json_encode($data->categories_selected);
      }elseif ($data->type == 'enteries' ){
        $offer->enteries = $data->enteries;
      }

      if($offer->save()){
        return response()->json(['status' => 200]);
      }
      return response()->json(['status' => 500]);
    }

    public function update_offer(Request $request, $id){
      $data = json_decode($request->offer);
      $offer = LoyaltyOffer::find($id);
      if($offer){
        $offer->name = $data->name;
        $offer->type = $data->type;
        if($data->type == 'off'){
          $offer->off_amount = $data->off_amount;
          $offer->off_order = $data->off_order;
        }elseif ($data->type == 'birthday_gift' || $data->type == 'christmas_gift' || $data->type == 'anniversary_gift' ){
          $offer->categories_selected = json_encode($data->categories_selected);
        }elseif ($data->type == 'enteries' ){
          $offer->enteries = $data->enteries;
        }
        if($offer->update()){
          return response()->json(['status' => 200]);
        }
        return response()->json(['status' => 500]);
      }
      return response()->json(['status' => 500]);
    }

    public function delete_offer($id){
      try {
        DB::beginTransaction();

        $offer = LoyaltyOffer::findOrFail($id);

        if ($offer->delete()) {
          DB::commit();
          return response()->json([
              'success' => true,
              'message' => 'Offer deleted.'
          ], 202);
        }
      } catch (QueryException $qe) {
        DB::rollback();

        return response()->json([
            'success' => false,
            'message' => $qe->getMessage()
        ], 500);
      } catch (\Exception $e) {
        DB::rollback();

        return response()->json([
            'success' => false,
            'message' => 'Offer not found.'
        ], 404);
      }
    }

    public function get_offers($type){
      if($type == 'all'){
        $offers = LoyaltyOffer::all();
        return response()->json($offers, 200);
      }
    }

    public function get_single_offer($id){
        $offer = LoyaltyOffer::find($id);
        if($offer->categories_selected != ''){
          $categories = ProductCategory::whereIn('id',json_decode($offer->categories_selected))->get();
          $offer->categories_selected = $categories;
        }
        return response()->json($offer, 200);
    }
}
