<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\OrdersStatusRequest;
use App\Http\Resources\OrdersResource;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use App\Models\Order;
use App\Models\OrderItem;
use App\Models\Paypal;
use App\User;
use Auth;

class OrdersController extends Controller
{
  protected $client;
  public function __construct(){
    $this->client = new ClientsController();
  }
    /**
     * Get orders with pagination
     *
     * @return  OrdersResource  orders with pagination data
     */
    public function index()
    {
        if (!empty($_GET['date'])) {
            if ($_GET['date'] == 'asc' || $_GET['date'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('created_at', $_GET['date'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        if (!empty($_GET['date_receive'])) {
            if ($_GET['date_receive'] == 'asc' || $_GET['date_receive'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('date_receive', $_GET['date_receive'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        if (!empty($_GET['payment_status'])) {
            if ($_GET['payment_status'] == 'asc' || $_GET['payment_status'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('payment_status', $_GET['payment_status'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        if (!empty($_GET['payment_type'])) {
            if ($_GET['payment_type'] == 'asc' || $_GET['payment_type'] == 'desc') {
                $clients = Order::with(['orderItems' => function ($query) {
                    $query->with(['product'])->get();
                }, 'orderClient'])->orderBy('payment_type', $_GET['payment_type'])->paginate(50);

                return OrdersResource::collection($clients);
            }
        }

        $orders = Order::with(['orderItems' => function ($query) {
            $query->with(['product'])->get();
        }, 'orderClient'])->paginate(50);
        return OrdersResource::collection($orders);
    //    $products = DB::table('ingredient_products')
    //         ->rightJoin('products', 'ingredient_products.product_id', '=', 'products.id')
    //         ->leftJoin('ingredients', 'ingredient_products.ingredient_id', '=', 'ingredients.id')
    //         ->select('products.name','ingredients.id')
    //         ->first();
    //         return response()->json([
    //             'success' => true,
    //             'data' => $products,
    //             OrdersResource::collection($orders),
    //             'message' => 'Products by categories fetch successfully.'
    //         ], 200);
    }

    /**
     * Get order details by ID
     *
     * @param   int  $id  order ID
     *
     * @return  JSON      order data or message
     */
    public function show($id)
    {
        try {
            $orders = Order::with(['orderItems' => function ($q) use ($id) {
                $q->with(['product' => function ($q1) use ($q, $id) {
                    $q1->with(['promoItem' => function ($q2) use ($q1, $q, $id) {
                        $q2->with(['promo'])->get();
                    }]);
                }])->get();
            }, 'orderClient'])->findOrFail($id);

            return response()->json([
                'data' => $orders,
            ]);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Order data not found.'
            ], 404);
        }
    }

    /**
     * Change order status
     *
     * @param   OrdersStatusRequest  $request  [$request description]
     *
     * @return  [type]                         [return description]
     */
    public function status(OrdersStatusRequest $request)
    {
        try {
            $id = $request->order_id;
            $status = $request->order_payment_status;

            $order = Order::with(['orderItems' => function ($query) {
                $query->with(['product'])->get();
            }, 'orderClient'])->findOrFail($id);

            if ($order->payment_type == 'cod' && $status == 'completed') {
                $order->payment_status = $status;
                $order->date_receive = Carbon::now();
            } else {
                $order->payment_status = $status;
            }

            if ($order->save()) {
                DB::commit();

                return response()->json([
                    'data' => $order,
                ]);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error while changing order status to database.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Change order status
     *
     * @param   OrdersStatusRequest  $request  [$request description]
     *
     * @return  [type]                         [return description]
     */
    public function saveorder(Request $request)
    {
        $orderID = $status = '';
        $cartArray = json_decode($request->cartArray);
        $validAddress = json_decode($request->validAddress);
        $user = $request->user;
        $totalAmount = $request->totalAmount;
        // $activePayment = $request->activePayment;

        switch ($request->activePayment) {
            case 0:
                $activePayment = 'credit_card';
                break;
            case 1:
                $activePayment = 'paypal';
                break;
            case 2:
                $activePayment = 'cod';
                break;
        }

        $order = new Order();
        $order->order_unique_id = date("YmdHis");
        $order->total_amount =  $request->finalTotal; //this is a final total amount
        $order->delivery_charge = 0;
        $order->payment_type = $activePayment;
        $order->sub_total = $totalAmount; //this is a sub total amount
        $order->discount_in_percent = number_format($request->discountInPercent);
        $order->promo_code = $request->promoCode;
        $order->payment_status = "new";
        $order->notes = "";
        $order->client_id = $user;
        $order->address1 = $request->address1;
        $order->address2 = $request->address2;
        $order->building_and_unit_number = $request->building_and_unit_number;
        $order->delivery_address = $request->delivery_address;
        $order->is_delivery_now = ($request->isDeliveryNow == "0") ? 1 : 0;
        $order->date_deliver = date('Y-m-d', strtotime($request->date_deliver));
        $order->time_deliver =  date('H:i:s', strtotime($request->time_deliver));
        $order->date_receive = Carbon::now();
        $order->created_at = Carbon::now();
        $order->updated_at = Carbon::now();
        $order->is_paid = 0;
/* ------- TALHA CODE STARTS HERE ---------- */
        if ($order->save()){
            $orderID = $order->id;
            $status = 200;

            foreach ($cartArray as $cart) {
                $selectedAllergies = array();
                $orderItem = new OrderItem();

                $orderItem->product_item_id = $cart->id;
                $orderItem->product_item_count = $cart->qty;
                if (!empty($cart->allergies)) {
                    foreach ($cart->allergies as $allergy) {
                        if ($allergy->selected) {
                            array_push($selectedAllergies, $allergy->name);
                        }
                    }
                }

                if (!empty($selectedAllergies)){
                    $orderItem->allergies = implode(',', $selectedAllergies);
                }else{
                    $orderItem->allergies = null;
                }


                $orderItem->order_id =  $order->id;
                $orderItem->save();
            }

            $user = User::find($user);
            $user->no_of_orders = (@$user->no_of_orders) ? @$user->no_of_orders + 1 : 1;
            $user->total_spent = $user->total_spent + $order->total_amount;
            $user->address1 = $request->address1;
            $user->phone=$request->phone_number;
            $user->save();
          $this->client->PaypalPaymentLogs(json_encode(['order_id' => $orderID,'status' => $status]) , $orderID , $request->activePayment , 'order_placement:200' , 'save_order');
        }else{
            $this->client->PaypalPaymentLogs(json_encode(['order_id' => null,'status' => 500]) , 0 , $request->activePayment , 'placement_failed' , 'save_order');
            $orderID = null;
            $status = 500;
        }
//        $paypal_data = json_decode(@$request->paypal, true);
//        if (count(@$paypal_data) != 0) {
//            $paypal = new Paypal;
//            $paypal->order_id = $order->id;
//            $paypal->cart = $paypal_data['cart'];
//            $paypal->create_time = date("Y-m-d H:i:s", strtotime($paypal_data['create_time']));
//            $paypal->paypal_id = $paypal_data['id'];
//            $paypal->intent = $paypal_data['intent'];
//            $paypal->payer = json_encode($paypal_data['payer']);
//            $paypal->state = $paypal_data['state'];
//            $paypal->transactions = json_encode($paypal_data['transactions']);
//            $paypal->save();
//        }



//        $arrResult = [];
//        $arrResult["order_id"] = $order->id;
//
//        echo json_encode($arrResult);
        return [
            'order_id' => $orderID,
            'status' => $status
        ];
/* ------- TALHA CODE ENDS HERE ---------- */
    }
/* ------- TALHA CODE STARTS HERE ---------- */
    public function savePaypalInfo(Request $request){
        $paypal_data = json_decode(@$request->paypal, true);
        if (count(@$paypal_data) != 0) {
            Order::find($request->orderID)->update(['is_paid' => '1']);
            $paypal = new Paypal;
            $paypal->order_id = $request->orderID;
            $paypal->cart = $paypal_data['cart'];
            $paypal->create_time = date("Y-m-d H:i:s", strtotime($paypal_data['create_time']));
            $paypal->paypal_id = $paypal_data['id'];
            $paypal->intent = $paypal_data['intent'];
            $paypal->payer = json_encode($paypal_data['payer']);
            $paypal->state = $paypal_data['state'];
            $paypal->transactions = json_encode($paypal_data['transactions']);
            $paypal->save();

            return [
                'status' => 200
            ];
        }
    }
/* ------- TALHA CODE ENDS HERE ---------- */

    /**
     * Get total orders count
     *
     * @return JSON      orders count data
     */
    public function totalOrdersStats()
    {
        $currStartMonth = Carbon::today()->startOfMonth();
        $currEndMonth = Carbon::today()->endOfMonth();
        $prevStartMonth = Carbon::now()->subMonth()->startOfMonth();
        $prevEndMonth = Carbon::now()->subMonth()->endOfMonth();

        $ordersCount = Order::get()->count();
        $ordersCountThisMonth = Order::whereBetween('created_at', [$currStartMonth, $currEndMonth])->count();
        $ordersCountPrevMonth = Order::whereBetween('created_at', [$prevStartMonth, $prevEndMonth])->count();
        $ordersTotalAmount = Order::sum('total_amount');
        $ordersTotalAmountThisMonth = Order::whereBetween('created_at', [$currStartMonth, $currEndMonth])->sum('total_amount');
        $ordersTotalAmountPrevMonth = Order::whereBetween('created_at', [$prevStartMonth, $prevEndMonth])->sum('total_amount');

        $ordersData = new \stdClass();
        $ordersData->count = $ordersCount;
        $ordersData->count_this_month = $ordersCountThisMonth;
        $ordersData->count_prev_month = $ordersCountPrevMonth;
        $ordersData->curr_percentage = ($ordersCountThisMonth === 0 && $ordersCountThisMonth === 0) ? 0 : $ordersCountThisMonth / $ordersCount * 100;
        $ordersData->prev_percentage = ($ordersCountPrevMonth === 0 && $ordersCountPrevMonth === 0) ? 0 : $ordersCountPrevMonth / $ordersCount * 100;
        $ordersData->total_amount = $ordersTotalAmount;
        $ordersData->curr_total_amount = $ordersTotalAmountThisMonth;
        $ordersData->prev_total_amount = $ordersTotalAmountPrevMonth;
        $ordersData->curr_total_amount_percentage = ($ordersTotalAmountThisMonth === 0 && $ordersTotalAmount === 0) ? 0 : $ordersTotalAmountThisMonth / $ordersTotalAmount * 100;
        $ordersData->prev_total_amount_percentage = ($ordersTotalAmountPrevMonth === 0 && $ordersTotalAmount === 0) ? 0 : $ordersTotalAmountPrevMonth / $ordersTotalAmount * 100;

        return response()->json($ordersData, 200);
    }

    /**
     * Get monthly total orders count
     *
     * @return JSON      orders count data
     */
    public function monthlyOrdersStats()
    {
        $orders = DB::table('orders')
            ->selectRaw('DATE_FORMAT(created_at, "%b") AS month, COUNT(*) AS total_orders, SUM(total_amount) AS total_amount')
            // ->where('created_at', '>', '"' . Carbon::now()->subMonths(5)->startOfMonth()->format('Y-m-d') . '"')
            ->groupBy(DB::raw('DATE_FORMAT(created_at, "%b")'))
            ->orderBy('id', 'desc')
            ->get();

        return response()->json($orders, 200);
    }

    public function myorderlist(Request $request)
    {
        $orders = new Order;
        $orders_data = $orders::select('*', DB::raw('DATE_FORMAT(created_at, "%d %b, %Y at  %H:%i:%s %p") as date'))
            ->where('client_id', $request->id)
            ->orderBy('id', 'desc')->get();
        return $orders_data;
    }
    public function countOrders(Request $request){
        $orders = new Order;
        $orders_data = $orders::count();
        return $orders_data;
    }

    public function ingredients(){
        $products = DB::table('order_item_ingredients')
        ->join('products', 'order_item_ingredients.product_item_id', '=', 'products.id')
        ->join('ingredients', 'order_item_ingredients.product_ingredient_id', '=', 'ingredients.id')
        ->select('order_item_ingredients.*','ingredients.name','ingredients.price')
        ->get();
        // $products = DB::table('ingredient_products')
        //     ->rightJoin('products', 'ingredient_products.product_id', '=', 'products.id')
        //     ->leftJoin('ingredients', 'ingredient_products.ingredient_id', '=', 'ingredients.id')
        //     ->select('products.id','ingredients.name','ingredients.price')
        //     ->get();
            return response()->json($products, 200);
    }
}
