<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\ProductCategoriesRequest;
use App\Http\Resources\ProductCategoriesResource;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Models\ProductCategory;
use App\Models\Product;
use Image;
use File;

class ProductCategoriesController extends Controller
{
    /**
     * Get product categories
     *
     * @return  JSON  data
     */
    public function index()
    {
        $categories = ProductCategory::paginate(100);

        return ProductCategoriesResource::collection($categories);
    }

    /**
     * Get all product categories without pagination
     *
     * @return  JSON  data
     */
    public function all()
    {
        $categories = ProductCategory::get();

        return response()->json($categories, 200);
    }

    public function parentCats()
    {
        $categories = ProductCategory::orderBy('sortable_id')
                                      ->get();

        return response()->json($categories, 200);
    }

    public function sortCategories(Request $request){
      $cats_sort = json_decode($request->sorting);
      foreach ($cats_sort as $cat) {
        //dd(intval($cat->id),$cat->sorted);
        ProductCategory::find(intval($cat->id))->update(['sortable_id' => $cat->sorted]);
      }
      return response()->json(200);
    }

    /**
     * Get all non-side menu product categories without pagination
     *
     * @return  JSON  data
     */
    public function main()
    {
          $categories = ProductCategory::
//          where('side_menu_type', '0')->orWhere('side_menu_type', '1')->orWhere('side_menu_type', '2')->
          get();

        return response()->json($categories, 200);
    }

    /**
     * Get all product categories without pagination
     *
     * @return  JSON  data
     */
    public function allproducts()
    {
        $categories = ProductCategory::
//        where('side_menu_type', '0')->orWhere('side_menu_type', '2')->
        get();
        $productsAll = Product::where('stock' , '>' , 0)->get();

        foreach ($categories as $category) {
            $products = [];
            if ($category->parent_id != 0 ){
                $products = Product::where('stock' , '>' , 0)->where("category_id", $category->id)->get();
            }
            $category->products =  $products;
            $category->activeClass = "inactive-menu";
        }
        $newObject = new \stdClass;
        $newObject->categories = $categories;
        $newObject->products = $productsAll;
        return json_encode($newObject);
    }

    /**
     * Get all side menu products with its catgories without pagination
     *
     * @return  JSON  data
     */
    public function sideMenu()
    {
        $productCategories = ProductCategory::with(['products'])
//            ->where('side_menu_type', '1')
            ->get();

        return response()->json($productCategories, 200);
    }

    /**
     * Get all side menu types with product categories without pagination
     *
     * @return  JSON  data
     */
    public function sideMenuTypes($id)
    {
        $productCategories = ProductCategory::with(['products'])->where('side_menu_type', '1')->get();

        return response()->json($productCategories, 200);
    }

    /**
     * Record product category
     *
     * @param   ProductCategoriesRequest  $request  request data
     *
     * @return  JSON                                success status and data or message
     */
    public function store(ProductCategoriesRequest $request)
    {
        $productCategory = new ProductCategory();

        if (!empty($request->category)) {
            $request->category = json_decode($request->category);
        } else {
            return response()->json([
                'message' => 'Category data enpty.'
            ], 422);
        }

        try {
            DB::beginTransaction();

            $productCategory->name = $request->category->category_name;
            $productCategory->side_menu_type = $request->category->side_menu_type;

            if (!empty($request->category->category_parent)){
                $productCategory->parent_id = $request->category->category_parent;
            }
            $productCategory->save();

            $updateProductCategory = ProductCategory::findOrFail($productCategory->id);
            if ($request->hasFile('category_desktop_pic')) {
                $desktopPicFile = $request->file('category_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt = pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/categories/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $productCategory->id . '.' . $desktopPicFileExt);
                $categoryDesktopPic = $productCategory->id . '.' . $desktopPicFileExt;
                $updateProductCategory->desktop_pic = (!empty($categoryDesktopPic)) ? $categoryDesktopPic : null;
                Image::make($request->file('category_desktop_pic')->getRealPath())->save($path . $categoryDesktopPic);
            }

            if ($request->hasFile('category_mobile_pic')) {
                $mobilePicFile = $request->file('category_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt = pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/categories/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $productCategory->id . '_m.' . $mobilePicFileExt);
                $categoryMobilePic = $productCategory->id . '_m.' . $mobilePicFileExt;
                $updateProductCategory->mobile_pic = (!empty($categoryMobilePic)) ? $categoryMobilePic : null;
                Image::make($request->file('category_mobile_pic')->getRealPath())->save($path . $categoryMobilePic);
            }
            if ($request->hasFile('category_header_pic')) {
                $headerPicFile = $request->file('category_header_pic');
                $headerPicFileName = $headerPicFile->getClientOriginalName();
                $headerPicFileExt = pathinfo($headerPicFileName, PATHINFO_EXTENSION);

                $path = 'img/categories/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $productCategory->id . '_h.' . $headerPicFileExt);
                $categoryheaderPic = $productCategory->id . '_h.' . $headerPicFileExt;
                $updateProductCategory->header_pic = (!empty($categoryheaderPic)) ? $categoryheaderPic : null;
                Image::make($request->file('category_header_pic')->getRealPath())->save($path . $categoryheaderPic);
            }
            if ($request->hasFile('category_mobile_header_pic')) {
                $mobileheaderPicFile = $request->file('category_mobile_header_pic');
                $mobileheaderPicFileName = $mobileheaderPicFile->getClientOriginalName();
                $mobileheaderPicFileExt = pathinfo($mobileheaderPicFileName, PATHINFO_EXTENSION);

                $path = 'img/categories/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $productCategory->id . '_mh.' . $mobileheaderPicFileExt);
                $categorymobileheaderPic = $productCategory->id . '_mh.' . $mobileheaderPicFileExt;
                $updateProductCategory->mobile_header_pic = (!empty($categorymobileheaderPic)) ? $categorymobileheaderPic : null;
                Image::make($request->file('category_mobile_header_pic')->getRealPath())->save($path . $categorymobileheaderPic);
            }



            if ($updateProductCategory->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $productCategory,
                    'data2' => $updateProductCategory,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error saving product category.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('category_desktop_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('category_mobile_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (\Exception $e) {
            DB::rollback();

            if ($request->hasFile('category_desktop_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('category_mobile_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->mobile_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Get product category details
     *
     * @param   int  $id  product category ID
     *
     * @return  Resource  product category details by ID
     */
    public function show($id)
    {
        try {
            $productCategory = ProductCategory::findOrFail($id);

            return response()->json($productCategory, 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Product category not found.'
            ], 404);
        }
    }

    /**
     * Update product category by ID
     *
     * @param   ProductCategoriesRequest  $request  request data
     *
     * @return  JSON                                success status and data or message
     */
    public function update(ProductCategoriesRequest $request, $id)
    {
        $productCategory = ProductCategory::findOrFail($id);

        if (!empty($request->category)) {
            $request->category = json_decode($request->category);
        } else {
            return response()->json([
                'message' => 'Category data enpty.'
            ], 422);
        }

        try {
            DB::beginTransaction();

            $productCategory->name = $request->category->category_name;
            $productCategory->side_menu_type = $request->category->side_menu_type;
            $productCategory->parent_id = $request->category->category_parent;
            $productCategory->save();

            $updateProductCategory = ProductCategory::findOrFail($productCategory->id);
            if ($request->hasFile('category_desktop_pic') && !empty($request->file('category_desktop_pic'))) {
                $desktopPicFile = $request->file('category_desktop_pic');
                $desktopPicFileName = $desktopPicFile->getClientOriginalName();
                $desktopPicFileExt = pathinfo($desktopPicFileName, PATHINFO_EXTENSION);

                $path = 'img/categories/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $productCategory->id . '.' . $desktopPicFileExt);
                $categoryDesktopPic = $productCategory->id . '.' . $desktopPicFileExt;
                $updateProductCategory->desktop_pic = $categoryDesktopPic;
                Image::make($request->file('category_desktop_pic')->getRealPath())->save($path . $categoryDesktopPic);
            }

            if ($request->hasFile('category_header_pic') && !empty($request->file('category_header_pic'))) {
                $headerPicFile = $request->file('category_header_pic');
                $headerPicFileName = $headerPicFile->getClientOriginalName();
                $headerPicFileExt = pathinfo($headerPicFileName, PATHINFO_EXTENSION);

                $path = 'img/categories/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $productCategory->id . '_h.' . $headerPicFileExt);
                $categoryHeaderPic = $productCategory->id . '_h.' . $headerPicFileExt;
                $updateProductCategory->header_pic = $categoryHeaderPic;
                Image::make($request->file('category_header_pic')->getRealPath())->save($path . $categoryHeaderPic);
            }
            if ($request->hasFile('category_mobile_pic') && !empty($request->file('category_mobile_pic'))) {
                $mobilePicFile = $request->file('category_mobile_pic');
                $mobilePicFileName = $mobilePicFile->getClientOriginalName();
                $mobilePicFileExt = pathinfo($mobilePicFileName, PATHINFO_EXTENSION);

                $path = 'img/categories/';

                if (!File::exists($path)) {
                    File::makeDirectory($path);
                }

                @unlink($path . $productCategory->id . '_m.' . $mobilePicFileExt);
                $categoryMobilePic = $productCategory->id . '_m.' . $mobilePicFileExt;
                $updateProductCategory->mobile_pic =  $categoryMobilePic;
                Image::make($request->file('category_mobile_pic')->getRealPath())->save($path . $categoryMobilePic);
            }

            if ($request->hasFile('category_mobile_header_pic') && !empty($request->file('category_mobile_header_pic'))) {
              $mobileheaderPicFile = $request->file('category_mobile_header_pic');
              $mobileheaderPicFileName = $mobileheaderPicFile->getClientOriginalName();
              $mobileheaderPicFileExt = pathinfo($mobileheaderPicFileName, PATHINFO_EXTENSION);

              $path = 'img/categories/';

              if (!File::exists($path)) {
                File::makeDirectory($path);
              }

              @unlink($path . $productCategory->id . '_mh.' . $mobileheaderPicFileExt);
              $categorymobileheaderPic = $productCategory->id . '_mh.' . $mobileheaderPicFileExt;
              $updateProductCategory->mobile_header_pic = (!empty($categorymobileheaderPic)) ? $categorymobileheaderPic : null;
              Image::make($request->file('category_mobile_header_pic')->getRealPath())->save($path . $categorymobileheaderPic);
            }


            if ($updateProductCategory->save()) {
                DB::commit();

                return response()->json([
                    'success' => true,
                    'data1' => $productCategory,
                    'data2' => $updateProductCategory,
                ], 200);
            } else {
                DB::rollback();

                return response()->json([
                    'success' => false,
                    'message' => 'Error updating product category.'
                ], 500);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            if ($request->hasFile('category_desktop_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('category_mobile_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->mobile_pic;
                \File::delete($file);
            }

            if ($request->hasFile('category_header_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->header_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (\Exception $e) {
            DB::rollback();

            if ($request->hasFile('category_desktop_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->desktop_pic;
                \File::delete($file);
            }

            if ($request->hasFile('category_mobile_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->mobile_pic;
                \File::delete($file);
            }

            if ($request->hasFile('category_header_pic')) {
                $file = public_path() . '/img/categories/' . $productCategory->header_pic;
                \File::delete($file);
            }

            return response()->json([
                'success' => false,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    /**
     * Destroy product category by ID
     *
     * @param   int  $id    product category ID to be deleted
     *
     * @return  JSON        success status with message
     */
    public function destroy($id)
    {
        try {
            DB::beginTransaction();

            $productCategory = ProductCategory::findOrFail($id);

            if ($productCategory->delete()) {
                DB::commit();

                $desktopImgFile = public_path() . '/img/categories/' . $productCategory->desktop_pic;
                \File::delete($desktopImgFile);

                $mobileImgFile = public_path() . '/img/categories/' . $productCategory->mobile_pic;
                \File::delete($mobileImgFile);

                return response()->json([
                    'success' => true,
                    'message' => 'Product category deleted.'
                ], 202);
            }
        } catch (QueryException $qe) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => $qe->getMessage()
            ], 500);
        } catch (\Exception $e) {
            DB::rollback();

            return response()->json([
                'success' => false,
                'message' => 'Product category not found.'
            ], 404);
        }
    }
}
