import Vue from 'vue';
import Vuex from 'vuex';
import {
    router
} from './routes.js';
import {
    BASE_URL
} from '../js/config.js';

Vue.use(Vuex);

export const store = new Vuex.Store({
    state: {
        isLoading: false,
        isLoggedin: false,
        isAdmin: false,
        user: null,
        settings: null,
        base_url: `${BASE_URL}`
    },
    actions: {
        reAuth: (context) => {
            let dateBackEndFormat = context.getters.getDateBackEndFormat;

            if (localStorage.getItem('mysushi.user') != null && dateBackEndFormat < localStorage.getItem('mysushi.expires_at')) {
                let user = JSON.parse(localStorage.getItem('mysushi.user'));
                context.commit('saveUser', user);
                context.commit('loginUser');
                context.state.isLoggedIn = true;
                context.commit('setAdminRights', (user.is_admin === '1'));
                let token = localStorage.getItem('mysushi.token');
                let config = {
                    headers: {
                        'Content-Type': 'application/json',
                        'Authorization': `Bearer ${token}`
                    }
                };

                context.commit('setAxiosConfig', config);

                axios.interceptors.request.use((config) => {
                    let token = localStorage.getItem('mysushi.token');

                    if (token) {
                        config.headers['Authorization'] = `Bearer ${token}`;
                        config.headers['Content-Type'] = `application/json`;
                    }

                    return config;
                }, (error) => {
                    return Promise.reject(error);
                });
            } else {

                context.commit.logout;
            }
        },
        logout: (context) => {
            let user = JSON.parse(localStorage.getItem('mysushi.user'));

            axios.get(context.getters.baseUrl + '/v1/logout', {
                    "headers": context.getters.axiosConfig.headers
                }).then(response => {
                    localStorage.removeItem("mysushi.token");
                    localStorage.removeItem("mysushi.user");
                    localStorage.removeItem("mysushi.token_type");
                    localStorage.removeItem("mysushi.expires_at");
                    localStorage.removeItem("cartArray");

                    context.commit('logoutUser');
                    context.commit('saveUser', null);

                    if (context.state.isAdmin) {
                        context.commit('setAdminRights', false);
                    }

                    router.push({
                        name: 'admin.auth'
                    });

                })
                .catch(error => {
                    console.log(error);
                });
        }
    },
    getters: {
        user: state => {
            return state.user;
        },
        settings: state => {
            return state.settings;
        },
        axiosConfig: state => {
            return state.axios_config;
        },
        loginStatus: state => {
            return state.isLoggedin;
        },
        adminRights: state => {
            return state.isAdmin;
        },
        baseUrl: state => {
            return state.base_url;
        },
        getDateBackEndFormat: state => {
            let date = new Date();
            let year = date.getFullYear();
            let month = date.getMonth() + 1;
            let day = date.getDate();
            let hours = date.getHours();
            let minutes = date.getMinutes();
            let seconds = date.getSeconds();

            return year + "-" + month + "-" + day + " " + hours + ":" + minutes + ":" + seconds;
        }
    },
    mutations: {
        loginUser: state => {
            state.isLoggedin = true;
        },
        logoutUser: state => {
            state.isLoggedin = false;
            state.user = null;
        },
        setAdminRights: (state, is_admin) => {
            state.isAdmin = is_admin;
        },
        saveUser: (state, user) => {
            state.user = user;
        },
        setSettings: (state, settings) => {
            state.settings = settings;
        },
        setLoading: (state, isLoading) => {
            state.isLoading = true;
        },
        unsetLoading: (state, isLaoding) => {
            state.isLoading = false
        },
        setAxiosConfig: (state, config) => {
            state.axios_config = config;
        },
    }
});
