<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::prefix('v1', ['middleware' => 'cors'])->group(function () {
    Route::post('auth', 'Api\AuthController@login');
    Route::post('fbAuth', 'Api\AuthController@fbAuth');
    Route::post('googleAuth', 'Api\AuthController@googleAuth');
    Route::post('register', 'Api\AuthController@register');
    Route::get('logout', 'Api\AuthController@logout');
    Route::get('/productCat', 'Api\ProductCategoriesController@main');
    Route::get('/ourBox', 'Api\OurBoxController@index');
    Route::get('/allproducts', 'Api\ProductCategoriesController@allproducts');
    Route::get('/products/categories/sidemenu', 'Api\ProductCategoriesController@sideMenu');
    Route::post('/newsLetter', 'Api\NewsLetterController@newsLetter');
    Route::post('/confirm', 'Api\MailController@confirm');
    Route::get('storage/app/{path}/{user_folder}/{filename}', function ($path, $user_folder, $filename) {
        $path = storage_path('app/' . $path . '/' . $user_folder . '/' . $filename);
        $file = File::get($path);
        $type = File::mimeType($path);
        $response = Response::make($file, 200);
        $response->header("Content-Type", $type);
        return $response;
    });

    Route::group([], function () {
        Route::get('logout', 'Api\AuthController@logout');
        Route::get('user', 'Api\AuthController@user');

        /**
         * Products-related Routes
         */
        Route::group(['prefix' => 'products'], function () {
            /**
             * Category-related Routes
             */
            Route::group(['prefix' => 'categories'], function () {
                Route::get('/', 'Api\ProductCategoriesController@index');
                Route::get('/all', 'Api\ProductCategoriesController@all');
                Route::get('/parents', 'Api\ProductCategoriesController@parentCats');

                Route::post('/store', 'Api\ProductCategoriesController@store');
                Route::post('/sortCategories', 'Api\ProductCategoriesController@sortCategories');
                Route::get('/{id}', 'Api\ProductCategoriesController@show');
                Route::put('/{id}/update', 'Api\ProductCategoriesController@update');
                Route::delete('/{id}/delete', 'Api\ProductCategoriesController@destroy');
            });

            Route::get('/', 'Api\ProductsController@index');
            Route::get('/by-category', 'Api\ProductsController@seriesByCategory');
            Route::post('/store', 'Api\ProductsController@store');
            Route::get('/{id}', 'Api\ProductsController@show');
            Route::put('/{id}/update', 'Api\ProductsController@update');
            Route::delete('/{id}/delete', 'Api\ProductsController@destroy');
        });

        /**
         * Promos-related Routes
         */
        Route::group(['prefix' => 'promos'], function () {
            Route::get('/', 'Api\PromosController@index');
            Route::get('/current', 'Api\PromosController@current');
            Route::post('/store', 'Api\PromosController@store');
            Route::get('/{id}', 'Api\PromosController@show');
            Route::put('/{id}/update', 'Api\PromosController@update');
            Route::delete('/{id}/delete', 'Api\PromosController@delete');
            Route::get('/trigger-active/{id}/{active}', 'Api\PromosController@triggerActiveStatus');
            Route::post('/promoCode', 'Api\PromosController@promoCode');
        });
        /**
         * Loyalty Program-related Routes
         */
        Route::group(['prefix' => 'loyaltyProgram'], function () {
            /*PROGRAMS*/
            Route::post('/program/store', 'Api\LoyaltyController@store_program');
            Route::post('/program/update/{id}', 'Api\LoyaltyController@update_program');
            Route::get('/program/get/{type}', 'Api\LoyaltyController@get_programs');
            Route::get('/program/{id}', 'Api\LoyaltyController@get_single_program');
            /*Route::delete('/program/{id}', 'Api\LoyaltyController@delete_offer');*/
            Route::get('/program/feature/{id}/{active}', 'Api\LoyaltyController@set_featured');
            Route::get('/program/status/{id}/{active}', 'Api\LoyaltyController@set_status');

          /*OFFERS*/
            Route::post('/offer/store', 'Api\LoyaltyController@store_offer');
            Route::post('/offer/update/{id}', 'Api\LoyaltyController@update_offer');
            Route::get('/offer/get/{type}', 'Api\LoyaltyController@get_offers');
            Route::get('/offer/{id}', 'Api\LoyaltyController@get_single_offer');
            Route::delete('/offer/{id}', 'Api\LoyaltyController@delete_offer');
        });

        /**
         * Clients-related Routes
         */
        Route::group(['prefix' => 'clients'], function () {
            Route::get('/', 'Api\ClientsController@index');
            Route::get('/{id}', 'Api\ClientsController@show');
            Route::post('/updateClientAccount', 'Api\ClientsController@updateClientAccount');
            Route::post('/saveAddress', 'Api\ClientsController@saveAddress');
            Route::post('/paypal_payment_logs', 'Api\ClientsController@PaypalPaymentLogs');
        });
        /**
         * Ingredients-related Routes
         */
        Route::group(['prefix'=>'ingredients'],function (){
            Route::get('/products', 'Api\IngredientsController@IngredientsProducts');
            Route::delete('/{id}/delete', 'Api\IngredientsController@destroy');
            Route::group(['prefix' => 'category'], function () {
                Route::get('/', 'Api\IngredientCategoryController@index');
                Route::post('/store', 'Api\IngredientCategoryController@store');
                Route::post('/status/{id}/change', 'Api\IngredientCategoryController@status_change');
                Route::get('/{id}', 'Api\IngredientCategoryController@show');
                Route::put('/{id}/update', 'Api\IngredientCategoryController@update');
                Route::delete('/{id}/delete', 'Api\IngredientCategoryController@destroy');
            });
            Route::get('/', 'Api\IngredientsController@index');
            Route::post('/store', 'Api\IngredientsController@store');
            Route::post('/status/{id}/change', 'Api\IngredientsController@status_change');
            Route::get('/{id}', 'Api\IngredientsController@show');
            Route::put('/{id}/update', 'Api\IngredientsController@update');
        });

        /**
         * Order-related Routes
         */
        Route::group(['prefix' => 'orders'], function () {
            Route::get('/ingredients', 'Api\OrdersController@ingredients');
            Route::get('/', 'Api\OrdersController@index');
            Route::get('/{id}', 'Api\OrdersController@show');
            Route::post('/status', 'Api\OrdersController@status');
            Route::post('/saveorder', 'Api\OrdersController@saveorder');
            Route::post('/savePaypalInfo', 'Api\OrdersController@savePaypalInfo');
            Route::post('/myorderlist', 'Api\OrdersController@myorderlist');
            Route::post('/countOrders', 'Api\OrdersController@countOrders');
        });

        Route::group(['prefix' => 'dashboards'], function () {
            Route::get('/ordersstats', 'Api\OrdersController@totalOrdersStats');
            Route::get('/monthlyordersstats', 'Api\OrdersController@monthlyOrdersStats');
            Route::get('/clientsstats', 'Api\ClientsController@clientsStats');
            Route::get('/productstopworststats', 'Api\ProductsController@topWorstItemsStats');
        });

        /**
         * Settings-related Routes
         */
        Route::group(['prefix' => 'settings'], function () {
            Route::get('/', 'Api\SettingsController@index');
            Route::get('/customization', 'Api\SettingsController@customization');
            Route::get('/customization/footer', 'Api\SettingsController@Footer');
            Route::post('/customization/update', 'Api\SettingsController@updateCustomization');
            Route::post('/customization/footer/update', 'Api\SettingsController@updateFooter');
            Route::post('/update', 'Api\SettingsController@update');
        });

        Route::group(['prefix' => 'locations'], function () {
            Route::get('/', 'Api\LocationController@index');
            Route::post('/status/{id}/change', 'Api\LocationController@status_change');
            Route::delete('{id}/delete', 'Api\LocationController@destroy');
            Route::post('/store', 'Api\LocationController@store');
        });
    });
});
